@@ -2,8 +2,9 @@
 
 import numpy as np
 from scipy import stats
-from typing import Callable, List, Union
+from typing import Callable, List, Sequence, Union
 
+from ..population import Sample
 from .base import Distance
 
 
@@ -42,20 +43,23 @@ class StochasticKernel(Distance):
     """
 
     def __init__(
-            self,
-            ret_scale: str = SCALE_LIN,
-            keys: List[str] = None,
-            pdf_max: float = None):
+        self,
+        ret_scale: str = SCALE_LIN,
+        keys: List[str] = None,
+        pdf_max: float = None,
+    ):
         StochasticKernel.check_ret_scale(ret_scale)
         self.ret_scale = ret_scale
         self.keys = keys
         self.pdf_max = pdf_max
 
     def initialize(
-            self,
-            t: int,
-            get_all_sum_stats: Callable[[], List[dict]],
-            x_0: dict = None):
+        self,
+        t: int,
+        get_sample: Callable[[], Sample],
+        x_0: dict,
+        total_sims: int,
+    ):
         """
         Remember the summary statistic keys in sorted order,
         if not set in __init__ already.
@@ -81,7 +85,6 @@ class SimpleFunctionKernel(StochasticKernel):
 
     Parameters
     ----------
-
     fun: Callable
         A Callable accepting `__call__`'s parameters.
         The function should be a pdf or pmf.
@@ -89,20 +92,22 @@ class SimpleFunctionKernel(StochasticKernel):
     """
 
     def __init__(
-            self,
-            fun: Callable,
-            ret_scale: str = SCALE_LIN,
-            keys: List[str] = None,
-            pdf_max: float = None):
+        self,
+        fun: Callable,
+        ret_scale: str = SCALE_LIN,
+        keys: List[str] = None,
+        pdf_max: float = None,
+    ):
         super().__init__(ret_scale=ret_scale, keys=keys, pdf_max=pdf_max)
         self.fun = fun
 
     def __call__(
-            self,
-            x: dict,
-            x_0: dict,
-            t: int = None,
-            par: dict = None) -> float:
+        self,
+        x: dict,
+        x_0: dict,
+        t: int = None,
+        par: dict = None,
+    ) -> float:
         return self.fun(x=x, x_0=x_0, t=t, par=par)
 
 
@@ -113,10 +118,8 @@ class NormalKernel(StochasticKernel):
 
     Parameters
     ----------
-
     cov: array_like, optional (default = identiy matrix)
         Covariance matrix of the distribution.
-
     ret_scale, keys, pdf_max: As in StochasticKernel.
 
 
@@ -128,25 +131,30 @@ class NormalKernel(StochasticKernel):
     """
 
     def __init__(
-            self,
-            cov: np.ndarray = None,
-            ret_scale: str = SCALE_LOG,
-            keys: List[str] = None,
-            pdf_max: float = None):
+        self,
+        cov: np.ndarray = None,
+        ret_scale: str = SCALE_LOG,
+        keys: List[str] = None,
+        pdf_max: float = None,
+    ):
         super().__init__(ret_scale=ret_scale, keys=keys, pdf_max=pdf_max)
         self.cov = cov
         self.ret_scale = ret_scale
 
     def initialize(
-            self,
-            t: int,
-            get_all_sum_stats: Callable[[], List[dict]],
-            x_0: dict = None):
+        self,
+        t: int,
+        get_sample: Callable[[], Sample],
+        x_0: dict,
+        total_sims: int,
+    ):
         # in particular set keys
         super().initialize(
             t=t,
-            get_all_sum_stats=get_all_sum_stats,
-            x_0=x_0)
+            get_sample=get_sample,
+            x_0=x_0,
+            total_sims=total_sims,
+        )
 
         # initialize distribution
         self._init_distr(x_0)
@@ -170,11 +178,12 @@ class NormalKernel(StochasticKernel):
         self.rv = stats.multivariate_normal(mean=mean, cov=self.cov)
 
     def __call__(
-            self,
-            x: dict,
-            x_0: dict,
-            t: int = None,
-            par: dict = None) -> float:
+        self,
+        x: dict,
+        x_0: dict,
+        t: int = None,
+        par: dict = None,
+    ) -> float:
         """
         Return the value of the normal distribution at x - x_0, or its
         logarithm.
@@ -218,23 +227,28 @@ class IndependentNormalKernel(StochasticKernel):
     """
 
     def __init__(
-            self,
-            var: Union[Callable, List[float], float] = None,
-            keys: List[str] = None,
-            pdf_max: float = None):
+        self,
+        var: Union[Callable, Sequence[float], float] = None,
+        keys: List[str] = None,
+        pdf_max: float = None,
+    ):
         super().__init__(ret_scale=SCALE_LOG, keys=keys, pdf_max=pdf_max)
         self.var = var
 
     def initialize(
-            self,
-            t: int,
-            get_all_sum_stats: Callable[[], List[dict]],
-            x_0: dict = None):
+        self,
+        t: int,
+        get_sample: Callable[[], Sample],
+        x_0: dict,
+        total_sims: int,
+    ):
         # in particular set keys
         super().initialize(
             t=t,
-            get_all_sum_stats=get_all_sum_stats,
-            x_0=x_0)
+            get_sample=get_sample,
+            x_0=x_0,
+            total_sims=total_sims,
+        )
 
         # dimension
         dim = sum(np.size(x_0[key]) for key in self.keys)
@@ -251,11 +265,12 @@ class IndependentNormalKernel(StochasticKernel):
             self.pdf_max = self(x_0, x_0)
 
     def __call__(
-            self,
-            x: dict,
-            x_0: dict,
-            t: int = None,
-            par: dict = None):
+        self,
+        x: dict,
+        x_0: dict,
+        t: int = None,
+        par: dict = None,
+    ):
         # safety check
         if self.keys is None:
             self.initialize_keys(x_0)
@@ -293,21 +308,18 @@ class IndependentLaplaceKernel(StochasticKernel):
 
     Parameters
     ----------
-
     scale: Union[array_like, float, Callable], optional (default = ones vector)
         Scale terms b of the distribution. Can also be a Callable taking as
         arguments the parameters. In that case, pdf_max should also be given
         if it is supposed to be used. Usually, it will then be given as the
         density at the observed statistics assuming the minimum allowed
         variance.
-
     keys, pdf_max: As in StochasticKernel.
-
     """
 
     def __init__(
             self,
-            scale: Union[Callable, List[float], float] = None,
+            scale: Union[Callable, Sequence[float], float] = None,
             keys: List[str] = None,
             pdf_max: float = None):
         super().__init__(ret_scale=SCALE_LOG, keys=keys, pdf_max=pdf_max)
@@ -315,15 +327,19 @@ class IndependentLaplaceKernel(StochasticKernel):
         self.dim = None
 
     def initialize(
-            self,
-            t: int,
-            get_all_sum_stats: Callable[[], List[dict]],
-            x_0: dict = None):
+        self,
+        t: int,
+        get_sample: Callable[[], Sample],
+        x_0: dict,
+        total_sims: int,
+    ):
         # in particular set keys
         super().initialize(
             t=t,
-            get_all_sum_stats=get_all_sum_stats,
-            x_0=x_0)
+            get_sample=get_sample,
+            x_0=x_0,
+            total_sims=total_sims,
+        )
 
         # dimension
         dim = sum(np.size(x_0[key]) for key in self.keys)
@@ -340,11 +356,12 @@ class IndependentLaplaceKernel(StochasticKernel):
             self.pdf_max = self(x_0, x_0)
 
     def __call__(
-            self,
-            x: dict,
-            x_0: dict,
-            t: int = None,
-            par: dict = None):
+        self,
+        x: dict,
+        x_0: dict,
+        t: int = None,
+        par: dict = None,
+    ):
         # safety check
         if self.keys is None:
             self.initialize_keys(x_0)
@@ -382,11 +399,12 @@ class BinomialKernel(StochasticKernel):
     """
 
     def __init__(
-            self,
-            p: Union[float, Callable],
-            ret_scale: str = SCALE_LOG,
-            keys: List[str] = None,
-            pdf_max: float = None):
+        self,
+        p: Union[float, Callable],
+        ret_scale: str = SCALE_LOG,
+        keys: List[str] = None,
+        pdf_max: float = None,
+    ):
         super().__init__(ret_scale=ret_scale, keys=keys, pdf_max=pdf_max)
 
         if not callable(p) and (p > 1 or p < 0):
@@ -396,15 +414,19 @@ class BinomialKernel(StochasticKernel):
         self.p = p
 
     def initialize(
-            self,
-            t: int,
-            get_all_sum_stats: Callable[[], List[dict]],
-            x_0: dict = None):
+        self,
+        t: int,
+        get_sample: Callable[[], Sample],
+        x_0: dict,
+        total_sims: int,
+    ):
         # in particular set keys
         super().initialize(
             t=t,
-            get_all_sum_stats=get_all_sum_stats,
-            x_0=x_0)
+            get_sample=get_sample,
+            x_0=x_0,
+            total_sims=total_sims,
+        )
 
         # cache pdf_max
         if self.pdf_max is None and not callable(self.p):
@@ -413,11 +435,12 @@ class BinomialKernel(StochasticKernel):
                 x_0, self.keys, self.p, self.ret_scale)
 
     def __call__(
-            self,
-            x: dict,
-            x_0: dict,
-            t: int = None,
-            par: dict = None) -> float:
+        self,
+        x: dict,
+        x_0: dict,
+        t: int = None,
+        par: dict = None,
+    ) -> float:
         x = np.asarray(_arr(x, self.keys), dtype=int)
         x_0 = np.asarray(_arr(x_0, self.keys), dtype=int)
 
@@ -442,22 +465,27 @@ class PoissonKernel(StochasticKernel):
     """
 
     def __init__(
-            self,
-            ret_scale: str = SCALE_LOG,
-            keys: List[str] = None,
-            pdf_max: float = None):
+        self,
+        ret_scale: str = SCALE_LOG,
+        keys: List[str] = None,
+        pdf_max: float = None,
+    ):
         super().__init__(ret_scale=ret_scale, keys=keys, pdf_max=pdf_max)
 
     def initialize(
-            self,
-            t: int,
-            get_all_sum_stats: Callable[[], List[dict]],
-            x_0: dict = None):
+        self,
+        t: int,
+        get_sample: Callable[[], Sample],
+        x_0: dict,
+        total_sims: int,
+    ):
         # in particular set keys
         super().initialize(
             t=t,
-            get_all_sum_stats=get_all_sum_stats,
-            x_0=x_0)
+            get_sample=get_sample,
+            x_0=x_0,
+            total_sims=total_sims,
+        )
 
         # cache pdf_max
         if self.pdf_max is None:
@@ -466,11 +494,12 @@ class PoissonKernel(StochasticKernel):
             self.pdf_max = self(x_0, x_0)
 
     def __call__(
-            self,
-            x: dict,
-            x_0: dict,
-            t: int = None,
-            par: dict = None) -> float:
+        self,
+        x: dict,
+        x_0: dict,
+        t: int = None,
+        par: dict = None,
+    ) -> float:
         x = np.asarray(_arr(x, self.keys), dtype=int)
         x_0 = np.asarray(_arr(x_0, self.keys), dtype=int)
 
@@ -495,11 +524,12 @@ class NegativeBinomialKernel(StochasticKernel):
     """
 
     def __init__(
-            self,
-            p: float,
-            ret_scale: str = SCALE_LOG,
-            keys: List[str] = None,
-            pdf_max: float = None):
+        self,
+        p: float,
+        ret_scale: str = SCALE_LOG,
+        keys: List[str] = None,
+        pdf_max: float = None,
+    ):
         super().__init__(ret_scale=ret_scale, keys=keys, pdf_max=pdf_max)
 
         if not callable(p) and (p > 1 or p < 0):
@@ -509,24 +539,29 @@ class NegativeBinomialKernel(StochasticKernel):
         self.p = p
 
     def initialize(
-            self,
-            t: int,
-            get_all_sum_stats: Callable[[], List[dict]],
-            x_0: dict = None):
+        self,
+        t: int,
+        get_sample: Callable[[], Sample],
+        x_0: dict,
+        total_sims: int,
+    ):
         # in particular set keys
         super().initialize(
             t=t,
-            get_all_sum_stats=get_all_sum_stats,
-            x_0=x_0)
+            get_sample=get_sample,
+            x_0=x_0,
+            total_sims=total_sims,
+        )
 
         # pdf_max is not computed
 
     def __call__(
-            self,
-            x: dict,
-            x_0: dict,
-            t: int = None,
-            par: dict = None) -> float:
+        self,
+        x: dict,
+        x_0: dict,
+        t: int = None,
+        par: dict = None,
+    ) -> float:
         x = np.asarray(_arr(x, self.keys), dtype=int)
         x_0 = np.asarray(_arr(x_0, self.keys), dtype=int)
 
