@@ -1,7 +1,12 @@
-from distributed import Client
+# TODO in python 3.9, the dask.distributed.Client() hangs if this is not
+#  imported first, until https://github.com/dask/distributed/issues/4168
+#  is fixed
+import multiprocessing.popen_spawn_posix  # noqa: F401
+import numpy as np
+from dask.distributed import Client
+
 from .base import Sampler
 from .eps_mixin import EPSMixin
-import numpy as np
 
 
 class DaskDistributedSampler(EPSMixin, Sampler):
@@ -12,17 +17,14 @@ class DaskDistributedSampler(EPSMixin, Sampler):
 
     Parameters
     ----------
-
     dask_client: dask.Client, optional
         The configured dask Client.
         If none is provided, then a local dask distributed cluster is created.
-
     client_max_jobs:
         Maximum number of jobs that can submitted to the client at a time.
         If this value is smaller than the maximum number of cores provided by
         the distributed infrastructure, the infrastructure will not be utilized
         fully.
-
     default_pickle:
         Specify if the sampler uses pythons default pickle function to
         communicate the submit function to python; if this is the case, a
@@ -31,14 +33,12 @@ class DaskDistributedSampler(EPSMixin, Sampler):
         functions, which can not be pickled using default pickle, at the cost
         of an additional pickling overhead. For dask, this workaround should
         not be necessary and it should be save to use default_pickle=false.
-
     batch_size: int, optional
         Number of parameter samples that are evaluated in one remote execution
         call. Batchsubmission can be used to reduce the communication overhead
         for fast (ms-s) model evaluations. Large batch sizes can result in un-
         necessary model evaluations. By default, batch_size=1, i.e. no
         batching is done.
-
     """
 
     def __init__(self, dask_client=None, client_max_jobs=np.inf,
@@ -70,8 +70,8 @@ class DaskDistributedSampler(EPSMixin, Sampler):
     def client_cores(self):
         return sum(self.my_client.ncores().values())
 
-    def stop(self):
-        """Close the dask client.
+    def shutdown(self):
+        """Shutdown the dask client.
         If it was started without arguments, the
         local cluster that was started at the same time is also closed.
         """
