@@ -79,7 +79,7 @@ def plot_kde_1d_highlevel(
         history: History, x: str, m: int = 0, t: int = None,
         xmin=None, xmax=None, numx=50, ax=None,
         size=None, title: str = None, refval=None, refval_color='C1',
-        kde=None, **kwargs):
+        kde=None, xname: str = None, **kwargs):
     """
     Plot 1d kernel density estimate of parameter samples.
 
@@ -120,6 +120,8 @@ def plot_kde_1d_highlevel(
         The kernel density estimator to use for creating a smooth density
         from the sample. If None, a multivariate normal kde with
         cross-validated scaling is used.
+    xname:
+        Parameter name. Defaults to `x`.
 
     Returns
     -------
@@ -129,12 +131,13 @@ def plot_kde_1d_highlevel(
     df, w = history.get_distribution(m=m, t=t)
 
     return plot_kde_1d(df, w, x, xmin, xmax, numx, ax, size, title, refval,
-                       refval_color, kde, **kwargs)
+                       refval_color, kde, xname, **kwargs)
 
 
-def plot_kde_1d(df, w, x, xmin=None, xmax=None,
-                numx=50, ax=None, size=None, title: str = None,
-                refval=None, refval_color='C1', kde=None, **kwargs):
+def plot_kde_1d(
+        df, w, x, xmin=None, xmax=None, numx=50, ax=None, size=None,
+        title: str = None, refval=None, refval_color='C1', kde=None,
+        xname: str = None, **kwargs):
     """
     Lowlevel interface for plot_kde_1d_highlevel (see there for the remaining
     parameters).
@@ -152,12 +155,14 @@ def plot_kde_1d(df, w, x, xmin=None, xmax=None,
         Axis of the plot.
     """
     x_vals, pdf = kde_1d(df, w, x, xmin=xmin, xmax=xmax,  numx=numx, kde=kde)
+    if xname is None:
+        xname = x
     if ax is None:
         _, ax = plt.subplots()
     ax.plot(x_vals, pdf, **kwargs)
     # TODO This fixes the upper bound inadequately
     # ax.set_ylim(bottom=min(ax.get_ylim()[0], 0))
-    ax.set_xlabel(x)
+    ax.set_xlabel(xname)
     ax.set_ylabel("Posterior")
     ax.set_xlim(xmin, xmax)
     if title is not None:
@@ -254,7 +259,8 @@ def plot_kde_2d_highlevel(
         xmin: float = None, xmax: float = None, ymin: float = None,
         ymax: float = None, numx: int = 50, numy: int = 50, ax=None,
         size=None, colorbar=True, title: str = None, refval=None,
-        refval_color='C1', kde=None, **kwargs):
+        refval_color='C1', kde=None, xname: str = None, yname: str = None,
+        **kwargs):
     """
     Plot 2d kernel density estimate of parameter samples.
 
@@ -307,7 +313,11 @@ def plot_kde_2d_highlevel(
     kde: pyabc.Transition, optional
         The kernel density estimator to use for creating a smooth density
         from the sample. If None, a multivariate normal kde with
-        cross-validated scaling is used..
+        cross-validated scaling is used.
+    xname:
+        Parameter name for the x-axis. Defaults to `x`.
+    xname:
+        Parameter name for the y-axis. Defaults to `y`.
 
     Returns
     -------
@@ -318,13 +328,13 @@ def plot_kde_2d_highlevel(
 
     return plot_kde_2d(
         df, w, x, y, xmin, xmax, ymin, ymax, numx, numy, ax, size, colorbar,
-        title, refval, refval_color, kde, **kwargs)
+        title, refval, refval_color, kde, xname, yname, **kwargs)
 
 
 def plot_kde_2d(df, w, x, y, xmin=None, xmax=None, ymin=None, ymax=None,
                 numx=50, numy=50, ax=None, size=None, colorbar=True,
                 title: str = None, refval=None, refval_color='C1', kde=None,
-                **kwargs):
+                xname: str = None, yname: str = None, **kwargs):
     """
     Plot a 2d kernel density estimate of parameter samples.
 
@@ -346,11 +356,15 @@ def plot_kde_2d(df, w, x, y, xmin=None, xmax=None, ymin=None, ymax=None,
                        xmin=xmin, xmax=xmax,
                        ymin=ymin, ymax=ymax, numx=numx, numy=numy,
                        kde=kde)
+    if xname is None:
+        xname = x
+    if yname is None:
+        yname = y
     if ax is None:
         _, ax = plt.subplots()
     mesh = ax.pcolormesh(X, Y, PDF, shading='auto', **kwargs)
-    ax.set_xlabel(x)
-    ax.set_ylabel(y)
+    ax.set_xlabel(xname)
+    ax.set_ylabel(yname)
     if title is not None:
         ax.set_title(title)
     if colorbar:
@@ -370,7 +384,7 @@ def plot_kde_matrix_highlevel(
         history, m: int = 0, t: int = None, limits=None,
         colorbar: bool = True, height: float = 2.5,
         numx: int = 50, numy: int = 50, refval=None, refval_color='C1',
-        kde=None, arr_ax=None):
+        kde=None, names: dict = None, arr_ax=None):
     """
     Plot a KDE matrix for 1- and 2-dim marginals of the parameter samples.
 
@@ -404,6 +418,8 @@ def plot_kde_matrix_highlevel(
         The kernel density estimator to use for creating a smooth density
         from the sample. If None, a multivariate normal kde with
         cross-validated scaling is used.
+    names:
+        Parameter names to use
     arr_ax:
         Array of axes objects to use.
 
@@ -416,12 +432,12 @@ def plot_kde_matrix_highlevel(
 
     return plot_kde_matrix(
         df, w, limits, colorbar, height, numx, numy, refval, refval_color,
-        kde, arr_ax)
+        kde, names, arr_ax)
 
 
 def plot_kde_matrix(df, w, limits=None, colorbar=True, height=2.5,
                     numx=50, numy=50, refval=None, refval_color='C1',
-                    kde=None, arr_ax=None):
+                    kde=None, names: dict = None, arr_ax=None):
     """
     Plot a KDE matrix for 1- and 2-dim marginals of the parameter samples.
 
@@ -441,8 +457,10 @@ def plot_kde_matrix(df, w, limits=None, colorbar=True, height=2.5,
     """
 
     n_par = df.shape[1]
-    par_names = list(df.columns.values)
+    par_ids = list(df.columns.values)
 
+    if names is None:
+        names = {key: key for key in par_ids}
     if arr_ax is None:
         fig, arr_ax = plt.subplots(nrows=n_par, ncols=n_par,
                                    sharex=False, sharey=False,
@@ -465,7 +483,7 @@ def plot_kde_matrix(df, w, limits=None, colorbar=True, height=2.5,
                     numx=numx, numy=numy,
                     ax=ax, title=None, colorbar=colorbar,
                     refval=refval, refval_color=refval_color,
-                    kde=kde)
+                    kde=kde, xname=names[x.name], yname=names[y.name])
 
     def scatter(x, y, ax):
         alpha = w / w.max()
@@ -484,11 +502,11 @@ def plot_kde_matrix(df, w, limits=None, colorbar=True, height=2.5,
                     xmax=limits.get(x.name, default)[1],
                     numx=numx,
                     ax=ax, refval=refval, refval_color=refval_color,
-                    kde=kde)
+                    kde=kde, xname=x.name)
 
     # fill all subplots
     for i in range(0, n_par):
-        y_name = par_names[i]
+        y_name = par_ids[i]
         y = df[y_name]
 
         # diagonal
@@ -496,7 +514,7 @@ def plot_kde_matrix(df, w, limits=None, colorbar=True, height=2.5,
         hist_1d(y, ax)
 
         for j in range(0, i):
-            x_name = par_names[j]
+            x_name = par_ids[j]
             x = df[x_name]
 
             # lower
@@ -508,7 +526,7 @@ def plot_kde_matrix(df, w, limits=None, colorbar=True, height=2.5,
             scatter(y, x, ax)
 
     # format
-    format_plot_matrix(arr_ax, par_names)
+    format_plot_matrix(arr_ax, [names[key] for key in par_ids])
 
     # adjust subplots to fit
     fig.tight_layout()
